/** file remove_bad_coords.c

Purpose:  This program removes bad trajectory frames from a certain series of MD runs.

Mechanisms:  
		(not for now) 1.  Any frame containing any entry that is precisely equal to zero is deleted. 
		
		2.  Any frame surviving condition 1 and containing a bond that is less than 
			MIN or greater than MAX Angstroms in length is deleted.

More info:  A RAID array experienced a minor malfunction during the runs.  Consequently, some
		data were corrupted.  All other frames appear to be ok.

*/

#include <glylib.h>

#define USAGE "\n\
        fix_coords input.parm7 input.crd output.crd deleted.crd logfile\n\
                Where:\n\
                    input.parm7 is the topology file\n\
                    input.crd is the trajectory containing bad frames\n\
                    output.crd is the new trajectory generated by this program\n\
                    deleted.crd contains any frames deleted by this program\n\
                    logfile is a file containing information generated by this program\n\
"
#define MIN  0.95*0.96  /** using SHAKE, so the H-bonds should not get much smaller */
#define MAX  1.15*1.53  /** 15% larger than 1.53 is probably too much */

int main (int argc, char *argv[])
{
int ai, /**< atom index */
    bi, /**< bond index */
    tsi, /**< trajectory set index */
    xfi, /**< coordinate format index */
    dbi, /**< deletions due to bond index */
    dzi, /**< deletions due to zeros index */
    tdi; /**< trajectory deletions count */
int test; /**< for file status */
char dropset='n', /**< flag to mark this set for being dropped */
     dropreason='n'; /**< flag to mark this set for being dropped */
molindex s,t;
assembly A;
fileset PF,/**< prmtop file */
	ICF, /**< trajcrd file in */
	OCF, /**< ok trajcrd file out */
	DCF, /**< deleted trajcrd file out */
	LOG; /**< log file */
vectormag_3D bondv;

//printf("1\n");

if(argc!=6) myusage(USAGE);

/** Open the log file and write initial information */
LOG.N=strdup(argv[5]);
LOG.F=myfopen(LOG.N,"w");
fprintf(LOG.F,"Starting trajcrd cleanup for these values:\n");
fprintf(LOG.F,"\tinput.parm7 = \"%s\"\n",argv[1]);
fprintf(LOG.F,"\tinput.crd = \"%s\"\n",argv[2]);
fprintf(LOG.F,"\toutput.crd = \"%s\"\n",argv[3]);
fprintf(LOG.F,"\tlogfile = \"%s\" (this file)\n\n",argv[4]);
fprintf(LOG.F,"The following information describes any trajectory deletions.\n\n");
fprintf(LOG.F,"%8s%8s%8s%8s%8s%8s%8s%8s%15s\n\n","#","Set #","Res1","Res#1","Atom1","Res2","Res#2","Atom2","Distance (A)");

PF.N=strdup(argv[1]);
ICF.N=strdup(argv[2]);
OCF.N=strdup(argv[3]);
DCF.N=strdup(argv[4]);

//printf("2\n");
/** Load the topology file */
A=load_amber_prmtop(PF); 
/** Open the input trajcrd file */
ICF.F=myfopen(ICF.N,"r");
/** Open the output trajcrd file */
OCF.F=myfopen(OCF.N,"w");
fprintf(OCF.F,"Trajectory modified by fix_coord from original file %s\n",ICF.N);
/** Open the deleted trajcrd file */
DCF.F=myfopen(DCF.N,"w");
fprintf(DCF.F,"Trajectory deletions by fix_coord from original file %s\n",ICF.N);

//printf("3\n");
tsi=tdi=dbi=dzi=0;
test=fgetc(ICF.F);
//printf("outside:  test is >>>%c<<< or %d\n",test,test);
while(test != EOF){
	ungetc(test,ICF.F);
	tsi++;
	/** Load the first/next frame in the coordinate file */
//printf("4\n");
	add_trajcrds_to_prmtop_assembly(ICF,&A,'c',0);
//printf("4.1\n");
	dropset='n';
	dropreason='n';
	for(ai=0;ai<A.na;ai++)
		{
		/** If the frame contains any entry that is precisely zero, clear everything and move on */
		/** Uncomment this block to make that happen
		if(A.a[ai][0].xa[0].i==0.000) dropset='y';
		if(A.a[ai][0].xa[0].j==0.000) dropset='y';
		if(A.a[ai][0].xa[0].k==0.000) dropset='y';
		if(dropset=='y')
			{
			dropreason='z';
			s=A.a[ai][0].moli;
			fprintf(LOG.F,"Trajectory set %d deleted.  Zero entry in %s(%d):%s.\n",tsi,\
				A.m[s.m][0].r[s.r].N,A.m[s.m][0].r[s.r].n,A.m[s.m][0].r[s.r].a[s.a].N);
			dzi++;
			break;
			}
		*/
		/** If the frame contains any bond that is less than MIN or more than MAX clear everything and move on */ 
		for(bi=0;bi<A.a[ai][0].nmb;bi++) 
			{
			s=A.a[ai][0].mb[bi].s;
			t=A.a[ai][0].mb[bi].t;
			if((A.m[s.m][0].r[s.r].a[s.a].t==-1)&&(A.m[t.m][0].r[t.r].a[t.a].t==-1)) continue; /* so not to double-count */
			A.m[s.m][0].r[s.r].a[s.a].t=A.m[t.m][0].r[t.r].a[t.a].t=-1;
			bondv=coord_to_vec(subtract_coord(A.m[t.m][0].r[t.r].a[t.a].xa[0],A.m[s.m][0].r[s.r].a[s.a].xa[0]));
			if((bondv.d<MIN)||(bondv.d>MAX)) dropset='y';
			}
		if(dropset=='y')
			{
			dropreason='b';
			dbi++;
			fprintf(LOG.F,"%8d%8d%8s%8d%8s%8s%8d%8s%15.2f\n",dbi,tsi,\
				A.m[s.m][0].r[s.r].N,A.m[s.m][0].r[s.r].n,A.m[s.m][0].r[s.r].a[s.a].N,\
				A.m[t.m][0].r[t.r].N,A.m[t.m][0].r[t.r].n,A.m[t.m][0].r[t.r].a[t.a].N,\
				bondv.d);
			break;
			}
		} 
	if(dropset=='y')
		{
		tdi++;
 		/** Write out the set that will be deleted */
		xfi=0;
		for(ai=0;ai<A.na;ai++)
			{
			fprintf(DCF.F,"%8.3f",A.a[ai][0].xa[0].i);
			xfi++;
			if((xfi%10)==0) fprintf(DCF.F,"\n");
			fprintf(DCF.F,"%8.3f",A.a[ai][0].xa[0].j);
			xfi++;
			if((xfi%10)==0) fprintf(DCF.F,"\n");
			fprintf(DCF.F,"%8.3f",A.a[ai][0].xa[0].k);
			xfi++;
			if((xfi%10)==0) fprintf(DCF.F,"\n");
			}
		if((xfi%10)!=0) fprintf(DCF.F,"\n");
		/** Now, reset everything for the next read */
		for(ai=0;ai<A.na;ai++)
			{
			free(A.a[ai][0].xa);
			A.a[ai][0].nalt=0;
			A.a[ai][0].t=0; /* set this back to zero */
			}	
		test=fgetc(ICF.F);
		continue;
		}
 	/** Still here?  Write out this coordinate set */
	xfi=0;
	for(ai=0;ai<A.na;ai++)
		{
		fprintf(OCF.F,"%8.3f",A.a[ai][0].xa[0].i);
		xfi++;
		if((xfi%10)==0) fprintf(OCF.F,"\n");
		fprintf(OCF.F,"%8.3f",A.a[ai][0].xa[0].j);
		xfi++;
		if((xfi%10)==0) fprintf(OCF.F,"\n");
		fprintf(OCF.F,"%8.3f",A.a[ai][0].xa[0].k);
		xfi++;
		if((xfi%10)==0) fprintf(OCF.F,"\n");
		}
	if((xfi%10)!=0) fprintf(OCF.F,"\n");
	/** Now, reset everything for the next read */
	for(ai=0;ai<A.na;ai++)
		{
		free(A.a[ai][0].xa);
		A.a[ai][0].nalt=0;
		A.a[ai][0].t=0; /* set this back to zero */
		}	
	test=fgetc(ICF.F);
//printf("near end:  test is >>>%c<<< or %d\n",test,test);
	}

fprintf(LOG.F,"\n\n======================= End of trajectory check =====================\n\n");
fprintf(LOG.F,"Number of trajectory frames scanned:  %d\n",tsi);
fprintf(LOG.F,"\tNumber of frames dropped:  %d\n",tdi);
//fprintf(LOG.F,"\t\tNumber of frames dropped due to a bad bond:  %d\n",dbi);
//fprintf(LOG.F,"\t\tNumber of frames dropped due to a zero entry:  %d\n",dzi);

return 0;
}
